// import { classes } from '@automapper/classes';
// import { AutomapperModule } from '@automapper/nestjs';
import { Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
// import { AuthorizationModule } from './modules/authorization/authorization.module';
// import { HealthModule } from './modules/health/health.module';
import { AuthenticationModule } from '@modules/authentication/authentication.module';
import { EventEmitterModule } from '@nestjs/event-emitter';
import {
  MessageBrokerModule,
  MessageBrokerModuleOptions,
} from '@mobicoop/message-broker-module';

@Module({
  imports: [
    ConfigModule.forRoot({ isGlobal: true }),
    EventEmitterModule.forRoot(),
    MessageBrokerModule.forRootAsync({
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (
        configService: ConfigService,
      ): Promise<MessageBrokerModuleOptions> => ({
        uri: configService.get<string>('MESSAGE_BROKER_URI'),
        exchange: configService.get<string>('MESSAGE_BROKER_EXCHANGE'),
        name: 'auth',
        handlers: {
          userUpdated: {
            routingKey: 'user.updated',
            queue: 'auth-user-updated',
          },
          userDeleted: {
            routingKey: 'user.deleted',
            queue: 'auth-user-deleted',
          },
        },
      }),
    }),
    // AutomapperModule.forRoot({ strategyInitializer: classes() }),
    AuthenticationModule,
    // AuthorizationModule,
    // HealthModule,
  ],
  controllers: [],
  providers: [],
})
export class AppModule {}
