import { Inject } from '@nestjs/common';
import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { DeleteUsernameCommand } from './delete-username.command';
import { USERNAME_REPOSITORY } from '@modules/authentication/authentication.di-tokens';
import { UsernameRepositoryPort } from '../../ports/username.repository.port';
import { UsernameEntity } from '@modules/authentication/core/domain/username.entity';

@CommandHandler(DeleteUsernameCommand)
export class DeleteUsernameService implements ICommandHandler {
  constructor(
    @Inject(USERNAME_REPOSITORY)
    private readonly usernameRepository: UsernameRepositoryPort,
  ) {}

  async execute(command: DeleteUsernameCommand): Promise<boolean> {
    const username: UsernameEntity = await this.usernameRepository.findOneById(
      command.name,
    );
    username.delete();
    const isDeleted: boolean = await this.usernameRepository.delete(username);
    return isDeleted;
  }
}
