import { HttpService } from '@nestjs/axios';
import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { lastValueFrom } from 'rxjs';
import { Action } from '../../domain/dtos/action.enum';
import { Domain } from '../../domain/dtos/domain.enum';
import { IMakeDecision } from '../../domain/interfaces/decision-maker';
import { ContextItem } from '../../domain/dtos/context-item';
import { Decision } from './decision';
import { Authorization } from '../../domain/entities/authorization';

@Injectable()
export class OpaDecisionMaker extends IMakeDecision {
  constructor(
    private readonly configService: ConfigService,
    private readonly httpService: HttpService,
  ) {
    super();
  }

  decide = async (
    domain: Domain,
    action: Action,
    context: Array<ContextItem>,
  ): Promise<Authorization> => {
    const reducedContext = context.reduce(
      (obj, item) => Object.assign(obj, { [item.name]: item.value }),
      {},
    );
    try {
      const { data } = await lastValueFrom(
        this.httpService.post<Decision>(
          this.configService.get<string>('OPA_URL') + domain + '/' + action,
          {
            input: {
              ...reducedContext,
            },
          },
        ),
      );
      return new Authorization(data.result.allow);
    } catch (e) {
      return new Authorization(false);
    }
  };
}
