import { Test, TestingModule } from '@nestjs/testing';
import { PrismaHealthIndicatorUseCase } from '../../domain/usecases/prisma.health-indicator.usecase';
import { AuthenticationRepository } from '../../../oldauthentication/adapters/secondaries/authentication.repository';
import { PrismaClientKnownRequestError } from '@prisma/client/runtime';
import { HealthCheckError, HealthIndicatorResult } from '@nestjs/terminus';

const mockAuthenticationRepository = {
  healthCheck: jest
    .fn()
    .mockImplementationOnce(() => {
      return Promise.resolve(true);
    })
    .mockImplementation(() => {
      throw new PrismaClientKnownRequestError('Service unavailable', {
        code: 'code',
        clientVersion: 'version',
      });
    }),
};

describe('PrismaHealthIndicatorUseCase', () => {
  let prismaHealthIndicatorUseCase: PrismaHealthIndicatorUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AuthenticationRepository,
          useValue: mockAuthenticationRepository,
        },
        PrismaHealthIndicatorUseCase,
      ],
    }).compile();

    prismaHealthIndicatorUseCase = module.get<PrismaHealthIndicatorUseCase>(
      PrismaHealthIndicatorUseCase,
    );
  });

  it('should be defined', () => {
    expect(prismaHealthIndicatorUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should check health successfully', async () => {
      const healthIndicatorResult: HealthIndicatorResult =
        await prismaHealthIndicatorUseCase.isHealthy('prisma');

      expect(healthIndicatorResult['prisma'].status).toBe('up');
    });

    it('should throw an error if database is unavailable', async () => {
      await expect(
        prismaHealthIndicatorUseCase.isHealthy('prisma'),
      ).rejects.toBeInstanceOf(HealthCheckError);
    });
  });
});
