import { CommandHandler } from '@nestjs/cqrs';
import { AuthenticationRepository } from '../../adapters/secondaries/authentication.repository';
import { CreateAuthenticationCommand } from '../../commands/create-authentication.command';
import { Authentication } from '../entities/authentication';
import * as bcrypt from 'bcrypt';
import { UsernameRepository } from '../../adapters/secondaries/username.repository';
import { Messager } from '../../adapters/secondaries/messager';

@CommandHandler(CreateAuthenticationCommand)
export class CreateAuthenticationUseCase {
  constructor(
    private readonly authenticationRepository: AuthenticationRepository,
    private readonly usernameRepository: UsernameRepository,
    private readonly messager: Messager,
  ) {}

  execute = async (
    command: CreateAuthenticationCommand,
  ): Promise<Authentication> => {
    const { uuid, password, ...username } = command.createAuthenticationRequest;
    const hash = await bcrypt.hash(password, 10);

    try {
      const auth = await this.authenticationRepository.create({
        uuid,
        password: hash,
      });

      await this.usernameRepository.create({
        uuid,
        ...username,
      });

      return auth;
    } catch (error) {
      this.messager.publish(
        'logging.auth.create.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
