import { CommandHandler } from '@nestjs/cqrs';
import { AuthenticationRepository } from '../../adapters/secondaries/authentication.repository';
import { Messager } from '../../adapters/secondaries/messager';
import { UsernameRepository } from '../../adapters/secondaries/username.repository';
import { DeleteAuthenticationCommand } from '../../commands/delete-authentication.command';
import { Authentication } from '../entities/authentication';

@CommandHandler(DeleteAuthenticationCommand)
export class DeleteAuthenticationUseCase {
  constructor(
    private readonly authenticationRepository: AuthenticationRepository,
    private readonly usernameRepository: UsernameRepository,
    private readonly messager: Messager,
  ) {}

  execute = async (
    command: DeleteAuthenticationCommand,
  ): Promise<Authentication> => {
    try {
      await this.usernameRepository.deleteMany({
        uuid: command.deleteAuthenticationRequest.uuid,
      });
      return await this.authenticationRepository.delete(
        command.deleteAuthenticationRequest.uuid,
      );
    } catch (error) {
      this.messager.publish(
        'logging.auth.delete.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
