import { UnauthorizedException } from '@nestjs/common';
import { CommandHandler } from '@nestjs/cqrs';
import { Messager } from '../../adapters/secondaries/messager';
import { UsernameRepository } from '../../adapters/secondaries/username.repository';
import { DeleteUsernameCommand } from '../../commands/delete-username.command';

@CommandHandler(DeleteUsernameCommand)
export class DeleteUsernameUseCase {
  constructor(
    private readonly usernameRepository: UsernameRepository,
    private readonly messager: Messager,
  ) {}

  execute = async (command: DeleteUsernameCommand): Promise<void> => {
    try {
      const { username } = command.deleteUsernameRequest;
      const usernameFound = await this.usernameRepository.findOne({
        username,
      });
      const usernames = await this.usernameRepository.findAll(1, 1, {
        uuid: usernameFound.uuid,
      });
      if (usernames.total > 1) {
        return await this.usernameRepository.deleteMany({ username });
      }
      throw new UnauthorizedException();
    } catch (error) {
      this.messager.publish(
        'logging.auth.username.delete.warning',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
